"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var testing_1 = require("@angular/core/testing");
var testing_2 = require("@angular/platform-browser-dynamic/testing");
var proxy_component_1 = require("./proxy.component");
var config_1 = require("./config");
var css_utils_1 = require("./css-utils");
var config = new config_1.CypressAngularConfig();
function setConfig(c) {
    config = c;
}
exports.setConfig = setConfig;
function init(component, moduleDef) {
    Cypress.log({ displayName: 'Unit Test', message: 'Init Environment' });
    checkIsComponentSpec();
    testing_1.TestBed.resetTestEnvironment();
    testing_1.TestBed.initTestEnvironment(testing_2.BrowserDynamicTestingModule, testing_2.platformBrowserDynamicTesting());
    var declarations = [component];
    var providers = [];
    // automatic component change detection
    if (config.detectChanges) {
        providers.push({ provide: testing_1.ComponentFixtureAutoDetect, useValue: true });
    }
    if (moduleDef) {
        if (moduleDef.declarations) {
            declarations.push.apply(declarations, moduleDef.declarations);
        }
        if (moduleDef.providers) {
            providers.push.apply(providers, moduleDef.providers);
        }
    }
    testing_1.TestBed.configureTestingModule({
        declarations: declarations,
        imports: moduleDef ? moduleDef.imports : [],
        providers: providers,
        schemas: moduleDef ? moduleDef.schemas : []
    });
    // @ts-ignore
    var document = cy.state('document');
    var el = document.getElementById('root');
    if (el === null) {
        throw new Error("root element not found");
    }
    css_utils_1.injectStylesBeforeElement(config, document, el);
}
;
function initEnv(component, moduleDef) {
    init(component, moduleDef);
    testing_1.TestBed.compileComponents();
}
exports.initEnv = initEnv;
;
function mount(component, inputs) {
    checkIsComponentSpec();
    // TODO improve logging using a full log instance
    Cypress.log({ displayName: 'Unit Test', message: "Mounting **" + component.name + "**" });
    var fixture = testing_1.TestBed.createComponent(component);
    var componentInstance = fixture.componentInstance;
    componentInstance = Object.assign(componentInstance, inputs);
    if (config.detectChanges) {
        fixture.whenStable().then(function () { return fixture.detectChanges(); });
        fixture.detectChanges();
    }
    return fixture;
}
exports.mount = mount;
;
function initEnvHtml(component, moduleDef) {
    if (moduleDef) {
        if (moduleDef.declarations) {
            moduleDef.declarations.push(component);
        }
        else {
            moduleDef.declarations = [component];
        }
    }
    else {
        moduleDef = { declarations: [component] };
    }
    init(proxy_component_1.ProxyComponent, moduleDef);
}
exports.initEnvHtml = initEnvHtml;
;
function mountHtml(htmlTemplate) {
    checkIsComponentSpec();
    Cypress.log({ displayName: 'Unit Test', message: "Mounting **" + htmlTemplate + "**" });
    testing_1.TestBed.overrideComponent(proxy_component_1.ProxyComponent, { set: { template: htmlTemplate } });
    testing_1.TestBed.compileComponents();
    var fixture = testing_1.TestBed.createComponent(proxy_component_1.ProxyComponent);
    if (config.detectChanges) {
        fixture.whenStable().then(function () { return fixture.detectChanges(); });
        fixture.detectChanges();
    }
    return fixture;
}
exports.mountHtml = mountHtml;
;
function getCypressTestBed() {
    return testing_1.getTestBed();
}
exports.getCypressTestBed = getCypressTestBed;
;
function checkIsComponentSpec() {
    if (!isComponentSpec()) {
        throw new Error('Angular component test from an integration spec is not allowed');
    }
}
;
// @ts-ignore
function isComponentSpec() { return Cypress.spec.specType === 'component'; }
